/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.urls;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Vector;

import org.testng.annotations.Test;

public class CustomUrlProviderTest
{

  private static final String cachedList = "TEST|http://someurl.blah/$DB_ACCESSION$|"
          + "ANOTHER|http://test/t$SEQUENCE_ID$|"
          + "TEST2|http://address/$SEQUENCE_ID$|SRS|"
          + "http://theSRSlink/$SEQUENCE_ID$";

  private static final HashMap<String, String> urlMap = new HashMap<String, String>()
  {
    {
      put("TEST","http://someurl.blah/$DB_ACCESSION$");
      put("ANOTHER","http://test/t$SEQUENCE_ID$");
      put("TEST2", "http://address/$SEQUENCE_ID$");
      put("SRS", "http://theSRSlink/$SEQUENCE_ID$");
    }
  };
  
  private static final String[] dlinks = {
      "TEST|http://someurl.blah/$DB_ACCESSION$",
      "ANOTHER|http://test/t$SEQUENCE_ID$",
      "TEST2|http://address/$SEQUENCE_ID$",
 UrlProviderI.DEFAULT_STRING };

  private static final Vector<String> displayLinks = new Vector<String>(
          Arrays.asList(dlinks));

  private static final String[] dlinks2 = { "a|http://x.y.z/$SEQUENCE_ID$" };

  private static final Vector<String> displayLinks2 = new Vector<String>(
          Arrays.asList(dlinks2));

  private static final String[] list1 = { "a" };

  private static final String[] list2 = { "http://x.y.z/$SEQUENCE_ID$" };

  private static final Vector<String> names = new Vector<String>(
          Arrays.asList(list1));

  private static final Vector<String> urls = new Vector<String>(
          Arrays.asList(list2));

  /*
   * Test default url is set and returned correctly
   */
  @Test(groups = { "Functional" })
  public void testDefaultUrl()
  {
    UrlProviderI customProv = new CustomUrlProvider(cachedList);
    
    // default url can be set
    assertTrue(customProv.setDefaultUrl("ANOTHER"));
    String result = customProv.getDefaultUrl();
    assertEquals("ANOTHER", result);

    // default url can be retrieved given a sequence id
    result = customProv.getDefaultUrl("seqid");
    assertEquals("http://test/tseqid", result);

    // if there is no default url it sets the default to null
    assertFalse(customProv.setDefaultUrl("No default"));
    result = customProv.getDefaultUrl();
    assertEquals(null, result);
    
    // choosing the default picks the DEFAULT_STRING option
    customProv.chooseDefaultUrl();
    result = customProv.getDefaultUrl("seqid");
    assertEquals(
            UrlProviderI.DEFAULT_STRING.split("\\|")[1].split("\\$")[0]
            + "seqid",
            result);
  }

  /*
   * Test urls are set and returned correctly
   */
  @Test(groups = { "Functional" })
  public void testUrlLinks()
  {
    // creation from cached url list works + old links upgraded
    UrlProviderI customProv = new CustomUrlProvider(cachedList);
    assertTrue(displayLinks.containsAll(customProv.getLinksForDisplay()));

    // creation from map works + old links upgraded
    UrlProviderI customProv2 = new CustomUrlProvider(urlMap);
    assertTrue(displayLinks.containsAll(customProv2.getLinksForDisplay()));

    // writing url links as a string works
    String result = customProv.writeUrlsAsString();
    UrlProviderI up = new CustomUrlProvider(result);
    assertTrue(displayLinks.containsAll(up.getLinksForDisplay()));

    result = customProv2.writeUrlsAsString();
    UrlProviderI up2 = new CustomUrlProvider(result);
    assertTrue(displayLinks.containsAll(up2.getLinksForDisplay()));

    // resetting urls from a pair of lists works
    customProv.setUrlLinks(names, urls);
    assertTrue(displayLinks2.containsAll(customProv.getLinksForDisplay()));

    customProv2.setUrlLinks(names, urls);
    assertTrue(displayLinks2.containsAll(customProv2.getLinksForDisplay()));
  }
}
